/***
 * Juliac
 * Copyright (C) 2008-2012 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliac;

import org.objectweb.fractal.juliac.conf.JDKLevel;

/**
 * Values recognized as command line options when invoking {@link Juliac}.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @since 1.2
 */
public enum CmdLineOptions {
	
	/**
	 * Dynamically loaded modules.
	 * @since 2.4
	 */
	MODULES(null),
    
    /**
     * Interceptor class generators.
     * @since 2.2.6
     */
    ICLASSGS(null),
    
    /** Interceptor source code generators. */
    ICODEGS(null),
    
    /**
     * Source files.
     * Semicolon-separated list of directories or jar files.
     */
    SRCS(null),
    
    /**
     * Mixins layer files.
     * Semicolon-separated list of directories or jar files.
     */
    MIXINS(null),
    
    /** Root package for generated code. */
    PKGROOT(null),
    
    /** Directory where generated source code is to be dumped. */
    GENSRC(null),
    
    /** Directory where compiled generated source code is to be dumped. */
    GENCLASS(null),
    
    /**
     * Jar file where compiled generated source code is to be packaged.
     * @since 2.2.4
     */
    GENCLASSJAR(null),
    
    /**
     * Source code compatibility level.
     * @since 2.2.2
     */
    SOURCE(JDKLevel.getVersions()),
    
    /**
     * Bytecode compatibility level.
     * @since 2.2.2
     */
    TARGET(JDKLevel.getVersions()),
    
    LOGLEVEL(null);
    
    private CmdLineOptions( String[] domain ) {
    	this.domain = domain;
    }
    
    /**
     * Return the domain values for the current constant or <code>null</code> if
     * no such domain is defined.
     * 
     * @since 2.2.4
     */
    public String[] getDomain() {
    	return domain;
    }
    private String[] domain;
}
