/***
 * Juliac
 * Copyright (C) 2007-2012 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliac;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import org.objectweb.fractal.juliac.api.CompileSupportItf;
import org.objectweb.fractal.juliac.conf.JuliacConfig;

/**
 * This class holds data about the input files which are compiled by Juliac.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 */
public class CompilationRound {
    
    public CompilationRound( CompilationRounds rounds, JuliacConfig jconf ) {
        this.rounds = rounds;
        this.jconf = jconf;
    }

    public void addInput( List<SourceFile> sfs ) {
        inputFiles.addAll(sfs);
    }
    
    public void addGenerated( SourceFile sf ) {
        generatedFiles.add(sf);
        Logger logger = jconf.getLogger();
        logger.info(sf.getQname());
    }
    
    /**
     * Return true if the specified file has already been generated in this
     * compilation round.
     */
    public boolean contains( SourceFile sf ) {
        boolean b = generatedFiles.contains(sf);
        return b;
    }
    
    /**
     * Compile the input files (source and generated).
     */
    public void compile( CompileSupportItf compiler )
    throws IOException {
        
        /*
         * Compile and move to the next compilation round.
         */
        List<SourceFile> sfs = new ArrayList<SourceFile>();
        sfs.addAll(inputFiles);
        sfs.addAll(generatedFiles);
        File classDir = jconf.getClassDir();
        outputFiles = compiler.compile(sfs,classDir);
        rounds.newCompilationRound();
    }
    
    public List<SourceFile> getInputFiles() { return inputFiles; }
    public List<SourceFile> getGeneratedFiles() { return generatedFiles; }
    public List<String> getOutputFileNames() { return outputFiles; }
    public File getClassDir() throws IOException {
        return jconf.getClassDir();
    }
    
    
    // --------------------------------------------------------------------
    // Data
    // --------------------------------------------------------------------

    /**
     * The source files which are given as input to Juliac.
     */
    private List<SourceFile> inputFiles = new ArrayList<SourceFile>();
    
    /**
     * The source files which are generated by Juliac.
     */
    private List<SourceFile> generatedFiles = new ArrayList<SourceFile>();
    
    /**
     * The names of .class files ouputted by the compilation process.
     */
    private List<String> outputFiles = new ArrayList<String>();
    
    
    // ------------------------------------------------------------------
    // Initialization data
    // ------------------------------------------------------------------
    
    private CompilationRounds rounds;
    private JuliacConfig jconf;
}
