/***
 * Juliac
 * Copyright (C) 2007-2012 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliac;

import java.util.ArrayList;
import java.util.List;

import org.objectweb.fractal.juliac.conf.JuliacConfig;

/**
 * This class manages the compilation rounds generated by Juliac.
 * Each compilation round contains a list of input files which need to be
 * compiled.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 */
public class CompilationRounds {
    
    public CompilationRounds( JuliacConfig jconf ) {
        this.jconf = jconf;
    }

    /**
     * Return a new compilation round.
     */
    public CompilationRound newCompilationRound() {
        CompilationRound round = new CompilationRound(this,jconf);
        rounds.add(round);
        return round;
    }
    
    /**
     * Return the compilation rounds.
     * 
     * @since 2.2.4
     */
    public List<CompilationRound> getCompilationRounds() {
    	return rounds;
    }
    
    /**
     * Return the current compilation round.
     */
    public CompilationRound getCurrentCompilationRound() {
        int idx = rounds.size();
        if( idx == 0 ) {
            CompilationRound round = newCompilationRound();
            return round;
        }
        CompilationRound round = rounds.get(idx-1);
        return round;
    }

    /**
     * Return <code>true</code> if the specified file has already been generated
     * in one of the compilation rounds associated with this instance.
     */
    public boolean contains( SourceFile sf ) {
        for (CompilationRound round : rounds) {
            boolean b = round.contains(sf);
            if(b) return true;
        }
        return false;
    }
    
    
    // --------------------------------------------------------------------
    // Data
    // --------------------------------------------------------------------

    private List<CompilationRound> rounds = new ArrayList<CompilationRound>();
    private JuliacConfig jconf;
}
