/***
 * Juliac
 * Copyright (C) 2007-2012 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliac.api;

import java.io.IOException;

import org.objectweb.fractal.juliac.visit.ClassSourceCodeVisitor;
import org.objectweb.fractal.juliac.visit.FileSourceCodeVisitor;

/**
 * Interface for class generators.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 */
public interface ClassGeneratorItf {

    /**
     * Generate the header of the source code file associated with this
     * generator. The header is the section which precedes the source code of
     * the class. The header may be composed of an initial comment, the
     * declaration of the package name and some import clauses.
     * 
     * @param fv  the visitor for generating the header of the source code file
     */
    public void generate( FileSourceCodeVisitor fv ) throws IOException;
    
    /**
     * Generate the source code of the class associated with this generator.
     * 
     * @param cv  the visitor for generating the source code of the class
     */
    public void generate( ClassSourceCodeVisitor cv ) throws IOException;

    /**
     * Return the name of the class generated by this generator.
     */
    public String getTargetClassName();
}
