/***
 * Juliac
 * Copyright (C) 2008-2012 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 * Author: Frederic Loiret
 */

package org.objectweb.fractal.juliac.api;

import java.io.IOException;
import java.util.List;

import org.objectweb.fractal.juliac.SourceFile;

/**
 * This interface describes the services provided by Spoon to Juliac.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @author Frederic Loiret <Frederic.Loiret@inria.fr>
 * @since 1.3
 */
public interface SpoonSupportItf extends JuliacModuleItf {

    /**
     * Mix the layers whose class names are specified and generate the resulting
     * mixed class.
     * 
     * @param targetClassName  the name of the generated mixed class
     * @param mixins           the list of layer class names
     * @throws IOException  in case of error during the code generation
     */
    public void mixAndGenerate( String targetClassName, List<String> mixins )
    throws IOException;

    /**
     * Return the Spoon factory. This method has been added to meet a
     * requirement from Hulotte which provides its own custom Spoon factory.
     * 
     * @author Frederic Loiret <Frederic.Loiret@inria.fr>
     * @since 2.2.2
     */
    public <T> T getSpoonFactory();
    
    
    /**
     * Build the Spoon code model corresponding to the specified source files.
     * 
     * @since 2.5
     */
    public void build( List<SourceFile> sfs ) throws IOException;

    /**
     * Dump the specified element in its corresponding source file. The element
     * is kept as an object although a CtSimpleType is expected. The rationale
     * is that we do not want to depend from Spoon in the juliac-core module. A
     * runtime type check is performed in the implementation of this interface
     * that is defined in the juliac-spoon module.
     * 
     * @since 2.5
     */
    public void generateSourceCode( Object element ) throws IOException;
}
