/***
 * Juliac
 * Copyright (C) 2010-2012 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliac.conf;

import org.objectweb.fractal.juliac.Juliac;
import org.objectweb.fractal.juliac.proxy.InterceptorSourceCodeGeneratorItf;

/**
 * This class holds data about interceptor source code generators.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @since 2.3
 */
public class InterceptorSourceCodeGeneratorMap
extends GeneratorMap<InterceptorSourceCodeGeneratorItf> {

	private static final long serialVersionUID = 3911547952035988597L;

	public InterceptorSourceCodeGeneratorMap( Juliac jc ) {
		super(jc);
	}

	/** @since 2.5 */
	@Override
	public InterceptorSourceCodeGeneratorMap clone() {
		InterceptorSourceCodeGeneratorMap clone =
			(InterceptorSourceCodeGeneratorMap) super.clone();
		return clone;
	}

    /**
     * Return the interceptor source code generators associated with the
     * specified class names.
     */
    public InterceptorSourceCodeGeneratorItf[]
        get( String[] asms, boolean mergeable ) {
        
    	InterceptorSourceCodeGeneratorItf[] scgs =
            new InterceptorSourceCodeGeneratorItf[asms.length];
        
        for (int i = 0; i < asms.length; i++) {            
        	String asm = asms[i];
            scgs[i] = get(asm);
            scgs[i].setJuliac(jc);
            scgs[i].setMergeable(mergeable);
        }
        
        return scgs;
    }	
}
