/***
 * Juliac
 * Copyright (C) 2009-2012 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliac.desc;

import java.util.HashMap;
import java.util.Map;

/**
 * Binding types (normal, export, import) as defined by {@link BindingBuilder}.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @since 2.2
 */
public enum BindingType {

    /** Normal binding. */
    NORMAL,
    
    /** Export binding. */
    EXPORT,
    
    /** Import binding. */
    IMPORT;

    /**
     * Return the binding type for the specified integer value in accordance
     * with the constant values defined in {@link BindingBuilder} for binding
     * types.
     * 
     * @param value  the integer value
     * @return       the corresponding binding type
     * @throws IllegalArgumentException
     *      if there is no such binding type for the specified value
     */
    public static BindingType getType( int value )
    throws IllegalArgumentException {
        
        if( ! map.containsKey(value) ) {
            String msg = "No binding type for value: "+value;
            throw new IllegalArgumentException(msg);
        }
        
        BindingType bt = map.get(value);
        return bt;
    }
    
    final private static Map<Integer,BindingType> map =
        new HashMap<Integer,BindingType>() {
            private static final long serialVersionUID = 5455962611496937975L;
        {
            put(0,NORMAL); // BindingBuilder.NORMAL_BINDING
            put(1,EXPORT); // BindingBuilder.EXPORT_BINDING
            put(2,IMPORT); // BindingBuilder.IMPORT_BINDING
        }};
}
