/***
 * Juliac
 * Copyright (C) 2008-2012 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 * Contributor: Romain Rouvoy
 */

package org.objectweb.fractal.juliac.proxy;

import java.lang.reflect.Modifier;

import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.juliac.Juliac;
import org.objectweb.fractal.juliac.Utils;
import org.objectweb.fractal.juliac.desc.MembraneDesc;
import org.objectweb.fractal.juliac.ucf.UnifiedClass;
import org.objectweb.fractal.juliac.ucf.UnifiedMethod;
import org.objectweb.fractal.juliac.visit.BlockSourceCodeVisitor;
import org.objectweb.fractal.juliac.visit.ClassSourceCodeVisitor;
import org.objectweb.fractal.juliac.visit.FileSourceCodeVisitor;

/**
 * Abstract implementation for interceptor source code generators.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @author Romain Rouvoy <Romain.Rouvoy@univ-lille1.fr>
 * @since 2.0
 */
public abstract class AbstractInterceptorSourceCodeGenerator
implements InterceptorSourceCodeGeneratorItf {

	protected Juliac jc;
    protected InterfaceType it;
    protected MembraneDesc<?> membraneDesc;
    protected boolean mergeable;
    
    public AbstractInterceptorSourceCodeGenerator() {}
    
    public AbstractInterceptorSourceCodeGenerator(
        Juliac jc,
    	InterfaceType it, MembraneDesc<?> membraneDesc, boolean mergeable ) {
        
    	setJuliac(jc);
        setInterfaceType(it);
        setMembraneDesc(membraneDesc);
        setMergeable(mergeable);
    }
    
    public void setJuliac( Juliac jc ) { this.jc = jc; }
    public void setInterfaceType( InterfaceType it ) { this.it = it; }
    public void setMembraneDesc( MembraneDesc<?> membraneDesc ) {
        this.membraneDesc = membraneDesc;
    }
    public void setMergeable( boolean mergeable ) { this.mergeable = mergeable; }

    /**
     * Return the name of the class generated by this generator.
     */
    public String getTargetClassName() {
        String javaItfSignature = it.getFcItfSignature();
        String name = Utils.getJuliacGeneratedStrongTypeName(javaItfSignature);
        String suffix = getClassNameSuffix();
        String targetClassname = name + suffix;
        return targetClassname;
    }    

    public String[] getClassGenericTypeParameters() {
        UnifiedClass proxycl = jc.create(it.getFcItfSignature());
        String[] tps = proxycl.getTypeParameters();
        return tps;
    }

    public int getClassModifiers() { return Modifier.PUBLIC; }
    public String getSuperClassName() { return null; }
    
    public String[] getImplementedInterfaceNames() {
        String signature = jc.getInterfaceTypeSignature(it);
        return new String[]{signature};
    }

    public void generateImports( FileSourceCodeVisitor fv ) {}
    public void generateStaticParts( ClassSourceCodeVisitor cv, UnifiedClass proxycl ) {}
    public void generateFields( ClassSourceCodeVisitor cv, UnifiedClass proxycl ) {}

    public void generateMethodInitFcController( BlockSourceCodeVisitor mv, UnifiedClass proxycl ) {}
    public void generateMethodClone( BlockSourceCodeVisitor mv, UnifiedClass proxycl ) {}
    public void generateMethodOthers( ClassSourceCodeVisitor cv, UnifiedClass proxycl ) {}
    
    public void generateProxyMethodBodyBeforeCode(
        BlockSourceCodeVisitor mv, UnifiedClass proxycl, UnifiedMethod proxym ) {}
    public void generateProxyMethodBodyAfterCode(
        BlockSourceCodeVisitor mv, UnifiedClass proxycl, UnifiedMethod proxym ) {}
    public void generateProxyMethodBodyFinallyCode(
        BlockSourceCodeVisitor mv, UnifiedClass proxycl, UnifiedMethod proxym ) {}

}
