/***
 * Juliac
 * Copyright (C) 2008-2012 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 * Contributor: Romain Rouvoy
 */

package org.objectweb.fractal.juliac.proxy;

import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.juliac.Juliac;
import org.objectweb.fractal.juliac.desc.MembraneDesc;
import org.objectweb.fractal.juliac.ucf.UnifiedClass;
import org.objectweb.fractal.juliac.ucf.UnifiedMethod;
import org.objectweb.fractal.juliac.visit.BlockSourceCodeVisitor;
import org.objectweb.fractal.juliac.visit.ClassSourceCodeVisitor;
import org.objectweb.fractal.juliac.visit.FileSourceCodeVisitor;

/**
 * Interface for interceptor source code generators.
 *
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille.fr>
 * @author Romain Rouvoy <Romain.Rouvoy@univ-lille1.fr>
 * @since 2.0
 */
public interface InterceptorSourceCodeGeneratorItf {
    
    /**
     * Return true if some code must be generated by this generator.
     */
    public boolean match();

    /**
     * Setter method for the instance of Juliac.
     */
    public void setJuliac( Juliac jc );
    
    /**
     * Setter method for the {@link InterfaceType} associated to this generator.
     */
    public void setInterfaceType( InterfaceType it );

    /**
     * Setter method for the membrane descriptor associated to this generator.
     */
    public void setMembraneDesc( MembraneDesc<?> membraneDesc );
    
    /**
     * Setter method for specifying whether the generated code can be merged. 
     */
    public void setMergeable( boolean mergeable );
    
    /**
     * Return the name of the suffix which is appended to generated class names.
     */
    public String getClassNameSuffix();
    
    public String[] getClassGenericTypeParameters();
    public int getClassModifiers();
    public String getSuperClassName();
    public String[] getImplementedInterfaceNames();

    public void generateImports( FileSourceCodeVisitor fv );
    public void generateStaticParts( ClassSourceCodeVisitor cv, UnifiedClass proxycl );
    public void generateFields( ClassSourceCodeVisitor cv, UnifiedClass proxycl );

    public void generateMethodInitFcController( BlockSourceCodeVisitor mv, UnifiedClass proxycl );
    public void generateMethodClone( BlockSourceCodeVisitor mv, UnifiedClass proxycl );
    public void generateMethodOthers( ClassSourceCodeVisitor cv, UnifiedClass proxycl );
    
    public void generateProxyMethodBodyBeforeCode(
        BlockSourceCodeVisitor mv, UnifiedClass proxycl, UnifiedMethod proxym );
    public void generateProxyMethodBodyAfterCode(
        BlockSourceCodeVisitor mv, UnifiedClass proxycl, UnifiedMethod proxym );
    public void generateProxyMethodBodyFinallyCode(
        BlockSourceCodeVisitor mv, UnifiedClass proxycl, UnifiedMethod proxym );
}
