/***
 * Juliac
 * Copyright (C) 2007-2012 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 * Contributor: Romain Rouvoy
 */

package org.objectweb.fractal.juliac.proxy;

import java.lang.reflect.Modifier;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Type;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.julia.BasicComponentInterface;
import org.objectweb.fractal.juliac.Juliac;
import org.objectweb.fractal.juliac.desc.MembraneDesc;
import org.objectweb.fractal.juliac.ucf.UnifiedClass;
import org.objectweb.fractal.juliac.ucf.UnifiedMethod;
import org.objectweb.fractal.juliac.visit.BlockSourceCodeVisitor;
import org.objectweb.fractal.juliac.visit.ClassSourceCodeVisitor;
import org.objectweb.fractal.juliac.visit.CodeHelper;

/**
 * This class generates the source code of Fractal interface implementations.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @author Romain Rouvoy <Romain.Rouvoy@univ-lille1.fr>
 */
public class InterfaceImplementationClassGenerator
extends AbstractProxyClassGenerator {

	public InterfaceImplementationClassGenerator(
		Juliac jc,
		InterfaceType it, MembraneDesc<?> membraneDesc, boolean mergeable ) {

		super(jc,it,membraneDesc,mergeable);
	}

	/**
	 * Return <code>true</code> if some code must be generated by the current
	 * source class generator.
	 */
	public boolean match() {
		return true;
	}

	/**
	 * Return the name of the suffix that is appended to generated class names.
	 */
	public String getClassNameSuffix() {
		return "FcItf";
	}

	@Override
	public void setMergeable(boolean mergeable) throws IllegalArgumentException {
		if (mergeable) {
			String msg = "This generator cannot generate mergeable code";
			throw new IllegalArgumentException(msg);
		}
	}

	// --------------------------------------------------------------------
	// ProxyClassGenerator methods
	// --------------------------------------------------------------------

	@Override
	public String getSuperClassName() {
		return BasicComponentInterface.class.getName();
	}

	@Override
	public String[] getImplementedInterfaceNames() {
		String signature = jc.getInterfaceTypeSignature(it);
		return new String[] { signature };
	}

	@Override
	public void generateFields(ClassSourceCodeVisitor cv, UnifiedClass proxycl) {
		generateFieldImpl(cv, proxycl);
	}

	@Override
	public void generateConstructors(ClassSourceCodeVisitor cv,
			UnifiedClass proxycl) {

		// public constructor()
		cv.visitConstructor(Modifier.PUBLIC, null, null, null).visitEnd();

		// public constructor(Component,String,Type,boolean,Object)
		String[] parameters = new String[] {
				Component.class.getName() + " component", "String s",
				Type.class.getName() + " type", "boolean flag", "Object obj" };
		BlockSourceCodeVisitor mv = cv.visitConstructor(Modifier.PUBLIC, null,
				parameters, null);
		mv.visitIns("super(component,s,type,flag,obj)").visitEnd();
	}

	@Override
	public void generateMethods(ClassSourceCodeVisitor cv, UnifiedClass proxycl) {
		generateMethodGetFcItfImpl(cv, proxycl);
		generateMethodSetFcItfImpl(cv, proxycl);
	}

	@Override
    public void generateProxyMethodBodyBeforeCode(
        BlockSourceCodeVisitor mv, UnifiedClass proxycl, UnifiedMethod proxym ) {
    	BlockSourceCodeVisitor then = mv.visitIf("impl","==","null");
    	then.visitIns(CodeHelper._throw(NullPointerException.class,
    			"\"Trying to invoke a method on a client or server interface whose complementary interface is not bound.\""));
    	then.visitEnd();
    }

	public String getDelegatingInstance(UnifiedClass proxycl,
			UnifiedMethod proxym) {
		return "impl";
	}

	// --------------------------------------------------------------------
	// InterfaceImplementationClassGenerator methods
	// --------------------------------------------------------------------

	public void generateFieldImpl(ClassSourceCodeVisitor cv,
			UnifiedClass proxycl) {
		String signature = jc.getInterfaceTypeSignature(it);
		cv.visitField(Modifier.PRIVATE, signature, "impl", null);
	}

	public void generateMethodGetFcItfImpl(ClassSourceCodeVisitor cv,
			UnifiedClass proxycl) {
		BlockSourceCodeVisitor mv = cv.visitMethod(Modifier.PUBLIC, null,
				"Object", "getFcItfImpl", null, null);
		mv.visitIns("return","impl").visitEnd();
	}

	public void generateMethodSetFcItfImpl(ClassSourceCodeVisitor cv,
			UnifiedClass proxycl) {
		BlockSourceCodeVisitor mv = cv.visitMethod(Modifier.PUBLIC, null,
				"void", "setFcItfImpl", new String[] { "Object obj" }, null);
		mv.visitSet("impl","("+jc.getInterfaceTypeSignature(it)+")obj").visitEnd();
	}
}
