/***
 * Juliac
 * Copyright (C) 2010-2012 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliac.ucf;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.Type;

/**
 * Implementation of {@link UnifiedField} for {@link Field}s.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @since 2.2.5
 */
public class UCField implements UnifiedField {
    
    /** The concrete {@link Field}. */
    private Field field;
    
    /** The class declaring the current field. */
    private UClass uClass;
    
    public UCField( UClass uClass, Field field ) {
    	this.uClass = uClass;
        this.field = field;
    }

	public UClass getDeclaringClass() {
		return uClass;
	}
	
    public <A extends Annotation> A getAnnotation( Class<A> annotationType ) {
        A a = field.getAnnotation(annotationType);
        return a;
    }

    public String getGenericType() {
        Type type = field.getGenericType();
        String typename = uClass.toString(type);
        return typename;
    }

    public int getModifiers() {
        int result = field.getModifiers();
        return result;
    }
    
    public String getName() {
        return field.getName();
    }

    public UnifiedClass getType() {
        Class<?> cl = field.getType();
        UnifiedClass uc = new UClass(cl);
        return uc;
    }

    @Override
    public String toString() {
        return field.toString();
    }
}
