/***
 * Juliac
 * Copyright (C) 2010-2012 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliac.ucf;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;

/**
 * This class implements an abstraction for {@link
 * spoon.reflect.declaration.CtField} and {@link Field}. The idea is to have
 * the possibility of manipulating transparently a field whatever its concrete
 * form (CtField or Field).
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @since 2.2.5
 */
public interface UnifiedField {
    
    /**
     * Return the field annotation for the specified annotation type,
     * or <code>null</code> if the field is not annotated.
     */
    public <A extends Annotation> A getAnnotation( Class<A> annotationType );

    /**
     * Return a string representation of the generic type for the current field.
     */
    public String getGenericType();
    
    /**
     * Return the modifiers of the current field.
     */
    public int getModifiers();
    
    /**
     * Return the name of the current field.
     */
    public String getName();
    
    /**
     * Return the return type of the current field.
     */
    public UnifiedClass getType();
    
    /**
     * Return the class declaring the current field.
     */
	public UnifiedClass getDeclaringClass();
}
