/***
 * Juliac
 * Copyright (C) 2007-2012 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliac.visit;

/**
 * A visitor for a file which contains some Java source code.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @since 1.1
 */
public interface FileSourceCodeVisitor {

    /**
     * Visit the source code file header. This header is printed before the
     * package keyword. This header must be a valid Java comment.
     * 
     * @param header  the header comment string
     */
    public void visitFileHeader( String header );
    
    /**
     * Visit the package name.
     * 
     * @param name  the package name
     */
    public void visitPackageName( String name );
    
    /**
     * Visit an import clause.
     * 
     * @param name  the imported type
     */
    public void visitImport( String name );
    
    /**
     * Visit the public class defined in this file.
     * 
     * @return  a visitor for visiting the source code of the public class
     */
    public ClassSourceCodeVisitor visitPublicClass();
}
